## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----dev_version_install, eval = FALSE----------------------------------------
#  BiocManager::install("ccb-hms/scDiagnostics")

## ----build_vignettes, eval=FALSE----------------------------------------------
#  BiocManager::install("ccb-hms/scDiagnostics",
#                       build_vignettes = TRUE,
#                       dependencies = TRUE)

## ----libraries, message = FALSE-----------------------------------------------
library(scDiagnostics)
library(scRNAseq)
library(scater)
library(scran)
library(scuttle)
library(SingleR)
library(AUCell)
library(celldex)

## ----QC-plot-QC-vs-Annotation-1, message=FALSE, warning=FALSE, eval=FALSE-----
#  # load reference dataset
#  ref_data <- fetchReference("hpca", "2024-02-26")
#  
#  # Load query dataset (Bunis haematopoietic stem and progenitor cell
#  # data) from Bunis DG et al. (2021). Single-Cell Mapping of
#  # Progressive Fetal-to-Adult Transition in Human Naive T Cells Cell
#  # Rep. 34(1): 108573
#  query_data <- BunisHSPCData()
#  rownames(query_data) <- rowData(query_data)$Symbol
#  
#  # Add QC metrics to query data
#  query_data <- addPerCellQCMetrics(query_data)
#  
#  # Log transform query dataset
#  query_data <- logNormCounts(query_data)
#  
#  # Run SingleR to predict cell types
#  pred <- SingleR(query_data, ref_data, labels = ref_data$label.main)
#  
#  # Assign predicted labels to query data
#  colData(query_data)$pred.labels <- pred$labels
#  
#  # Get annotation scores
#  scores <- apply(pred$scores, 1, max)
#  
#  # Assign scores to query data
#  colData(query_data)$cell_scores <- scores
#  
#  # Create a scatter plot between library size and annotation scores
#  p1 <- plotQCvsAnnotation(query_data = query_data,
#                           qc_col = "total",
#                           label_col = "pred.labels",
#                           score_col = "cell_scores",
#                           label = NULL)
#  p1 + xlab("Library Size")

## ----QC-plot-QC-vs-Annotation-2, warning=FALSE, message=FALSE, eval=FALSE-----
#  # load query dataset
#  query_data <- HeOrganAtlasData(tissue = c("Liver"), ensembl = FALSE, location = TRUE)
#  
#  # Add QC metrics to query data
#  
#  mito_genes <- rownames(query_data)[grep("^MT-", rownames(query_data))]
#  query_data <- unfiltered <- addPerCellQC(query_data,subsets = list(mt = mito_genes))
#  qc <- quickPerCellQC(colData(query_data), sub.fields = "subsets_mt_percent")
#  query_data <- query_data[,!qc$discard]
#  
#  # Log transform query dataset
#  query_data <- logNormCounts(query_data)
#  
#  # Run SingleR to predict cell types
#  pred <- SingleR(query_data, ref_data, labels = ref_data$label.main)
#  
#  # Assign predicted labels to query data
#  colData(query_data)$pred.labels <- pred$labels
#  
#  # Get annotation scores
#  scores <- apply(pred$scores, 1, max)
#  
#  # Assign scores to query data
#  colData(query_data)$cell_scores <- scores
#  
#  # Create a new column for the labels so it is easy to distinguish
#  #  between Macrophoges, Monocytes and other cells
#  query_data$label_category <-
#      ifelse(query_data$pred.labels %in% c("Macrophage", "Monocyte"),
#             query_data$pred.labels,
#             "Other Cells")
#  
#  
#  # Define custom colors for cell type labels
#  cols <- c("Other Cells" = "grey", "Macrophage" = "green", "Monocyte" = "red")
#  
#  # Generate scatter plot for all cell types
#  p2 <- plotQCvsAnnotation(query_data = query_data,
#                           qc_col = "subsets_mt_percent",
#                           label_col = "label_category",
#                           score_col = "cell_scores",
#                           label = NULL)
#  p2 + scale_color_manual(values = cols) +
#      xlab("Subsets Mitochondrial Percentage (%)")

## ----hist-QC-vs-Annotation, warning=FALSE, message=FALSE, eval=FALSE----------
#  # Generate histogram
#  histQCvsAnnotation(query_data = query_data, qc_col = "subsets_mt_percent",
#                     label_col = "pred.labels",
#                     score_col = "cell_scores",
#                     label = NULL)

## ----plot-Marker-Expression, warning=FALSE, message=FALSE---------------------
# Load data
sce <- HeOrganAtlasData(tissue = c("Marrow"), ensembl = FALSE)

# Divide the data into reference and query datasets
set.seed(100)
indices <- sample(ncol(assay(sce)), size = floor(0.7 * ncol(assay(sce))), replace = FALSE)
ref_data <- sce[, indices]
query_data <- sce[, -indices]

# Log-transform datasets
ref_data <- logNormCounts(ref_data)
query_data <- logNormCounts(query_data)

# Get cell type scores using SingleR
pred <- SingleR(query_data, ref_data, labels = ref_data$reclustered.broad)

# Assign labels to query data
colData(query_data)$labels <- pred$labels
   
# Generate density plots
plotMarkerExpression(reference_data = ref_data, 
                     query_data = query_data, 
                     ref_cell_type_col = "reclustered.broad", 
                     query_cell_type_col = "labels", 
                     gene_name = "MS4A1", 
                     label = "B_and_plasma")

## ----plot-Gene-Expression-Dimred, warning=FALSE, message=FALSE----------------
# Run PCA on the query data
query_data <- runPCA(query_data)

# Generate dimension reduction plot color code by gene expression
# plotGeneExpressionDimred(se_object = query_data, 
#                          method = "PCA", 
#                          pc_subset = c(1:5), 
#                          feature = "VPREB3")

## ----plot-Gene-Set-Scores, warning=FALSE, message=FALSE, eval=FALSE, eval=FALSE----
#  # Compute scores using AUCell
#  expression_matrix <- assay(query_data, "logcounts")
#  cells_rankings <- AUCell_buildRankings(expression_matrix, plotStats = FALSE)
#  
#  # Generate gene sets
#  gene_set1 <- sample(rownames(expression_matrix), 10)
#  gene_set2 <- sample(rownames(expression_matrix), 20)
#  
#  gene_sets <- list(geneSet1 = gene_set1,
#                    geneSet2 = gene_set2)
#  
#  # Calculate AUC scores for gene sets
#  cells_AUC <- AUCell_calcAUC(gene_sets, cells_rankings)
#  
#  # Assign scores to colData
#  colData(query_data)$geneSetScores <- assay(cells_AUC)["geneSet1", ]
#  
#  # Plot gene set scores on PCA
#  plotGeneSetScores(se_object = query_data,
#                    method = "PCA",
#                    feature = "geneSetScores",
#                    pc_subset = c(1:6))

## ----calculate-HVG-Overlap, warning=FALSE, message=FALSE----------------------
# Get top HVG genes on each dataset
ref_var <- getTopHVGs(ref_data, n = 2000) 
query_var <- getTopHVGs(query_data, n = 2000)  

# Compute the overlap coefficient 
overlap_coefficient <- calculateHVGOverlap(reference_genes = ref_var,
                                           query_genes = query_var) 
overlap_coefficient

## ----calculate-Var-Imp-Overlap, warning=FALSE, message=FALSE------------------
# Intersect the gene symbols to obtain common genes
common_genes <- intersect(ref_var, query_var)

# Select desired cell types
selected_cell_types <- c("CD4", "CD8", "B_and_plasma")
ref_data_subset <- ref_data[common_genes, ref_data$reclustered.broad %in% selected_cell_types]
query_data_subset <- query_data[common_genes, query_data$labels %in% selected_cell_types]

# Compute important variables for all pairwise cell comparisons
var_imp_overlap <- calculateVarImpOverlap(ref_data_subset, 
                                          query_data_subset,
                                          ref_cell_type_col = "reclustered.broad", 
                                          query_cell_type_col = "labels",
                                          n_top = 50)

# Comparison table
var_imp_overlap$var_imp_comparison

## ----visualize-Cell-Type-MDS, warning=FALSE, message=FALSE--------------------
# Generate the MDS scatter plot with cell type coloring
visualizeCellTypeMDS(query_data = query_data_subset, 
                     reference_data = ref_data_subset, 
                     query_cell_type_col = "labels",
                     ref_cell_type_col = "reclustered.broad")

## ----visualize-Cell-Type-PCA, warning=FALSE, message=FALSE, eval=FALSE--------
#  # Run PCA on the reference data
#  ref_data_subset <- runPCA(ref_data_subset)
#  
#  # Plot the PCs data
#  visualizeCellTypePCA(query_data = query_data_subset, reference_data = ref_data_subset,
#                       query_cell_type_col = "reclustered.broad",
#                       ref_cell_type_col = "reclustered.broad",
#                       pc_subset = c(1:6))

## ----boxplot-PCA, warning=FALSE, message=FALSE--------------------------------
# Run PCA on the reference data
ref_data_subset <- runPCA(ref_data_subset)

# Boxplots of PCs
boxplotPCA(query_data = query_data_subset, reference_data = ref_data_subset,
           query_cell_type_col = "labels", 
           ref_cell_type_col = "reclustered.broad", 
           pc_subset = c(1:6))

## ----calculate-Hotelling-P-Value, warning=FALSE, message=FALSE----------------
p_values <- calculateHotellingPValue(query_data = query_data_subset, 
                                     reference_data = ref_data_subset, 
                                     query_cell_type_col = "labels", 
                                     ref_cell_type_col = "reclustered.broad",
                                     pc_subset = c(1:6)) 
round(p_values, 5)

## ----regress-PC, warning=FALSE, message=FALSE---------------------------------
# Specify the dependent variables (principal components) and
#  independent variable (e.g., "labels")
dep.vars <- paste0("PC", 1:12)
indep.var <- "labels"

# Perform linear regression on multiple principal components
result <- regressPC(sce = query_data,
                    dep.vars = dep.vars, 
                    indep.var = indep.var)

# Print the summaries of the linear regression models and R-squared
#  values

# Summaries of the linear regression models
result$regression.summaries[[1]]

# R-squared values
result$rsquared

# Variance contributions for each principal component
result$var.contributions

# Total variance explained
result$total.variance.explained

## -----------------------------------------------------------------------------
pc_df <-  attr(reducedDims(query_data)$PCA, "rotation")[, 1:5] |> 
  as.data.frame()

pc_df[order(abs(pc_df$PC1)), "PC1", drop = FALSE] |> 
  tail()

## ----calculate-Average-Pairwise-Correlation, warning=FALSE, message=FALSE-----
cor_matrix_avg <- calculateAveragePairwiseCorrelation(query_data = query_data_subset,
                                                      reference_data = ref_data_subset, 
                                                      query_cell_type_col = "labels", 
                                                      ref_cell_type_col = "reclustered.broad", 
                                                      cell_types = selected_cell_types, 
                                                      correlation_method = "spearman")

# Visualize the output
plot(cor_matrix_avg)

## ----plotPairwise-Distances-Density, fig.width=8, message=FALSE, warning=FALSE----
plotPairwiseDistancesDensity(query_data = query_data_subset, 
                             reference_data = ref_data_subset, 
                             query_cell_type_col = "labels", 
                             ref_cell_type_col = "reclustered.broad", 
                             cell_type_query = "CD4", 
                             cell_type_reference = "CD4", 
                             distance_metric = "euclidean")

## ----plot-Pairwise-Distances-Density, warning=FALSE, message=FALSE, fig.width=8----
plotPairwiseDistancesDensity(query_data = query_data_subset, 
                             reference_data = ref_data_subset, 
                             query_cell_type_col = "labels", 
                             ref_cell_type_col = "reclustered.broad", 
                             cell_type_query = "CD4", 
                             cell_type_reference = "CD4", 
                             distance_metric = "correlation",
                             correlation_method = "spearman")

## ----nearest-Neighbor-Diagnostics, warning=FALSE, message=FALSE, fig.width=8----
nn_output <- calculateNearestNeighborProbabilities(query_data = query_data_subset, 
                                                   reference_data = ref_data_subset,
                                                   query_cell_type_col = "labels", 
                                                   ref_cell_type_col = "reclustered.broad")

# Plot output
plot(nn_output)

## ----detect-Anomaly, warning=FALSE, message=FALSE, fig.width=8----------------
# Store PCA anomaly data
anomaly_output <- detectAnomaly(reference_data = ref_data_subset, 
                                query_data = query_data_subset, 
                                ref_cell_type_col = "reclustered.broad", 
                                query_cell_type_col = "labels") 

# Plot the anomaly output for a cell type
# plot(anomaly_output, cell_type = "CD4", pc_subset = c(1:5), data_type = "query")

## ----calculate-Sample-Distances, warning=FALSE, message=FALSE, fig.width=8----
# Compute sample distance data
distance_data <- calculateSampleDistances(query_data = query_data_subset, reference_data = ref_data_subset, 
                                          query_cell_type_col = "labels", 
                                          ref_cell_type_col = "reclustered.broad") 

# Store top six anomalies for CD4
cd4_top6_anomalies <- names(sort(anomaly_output$CD4$query_anomaly_scores, decreasing = TRUE)[1:6])

# Plot the densities of the distances for the anomalies
plot(distance_data, ref_cell_type = "CD4", sample_names = cd4_top6_anomalies)

## ----calculate-Sample-Distances-CD8, warning=FALSE, message=FALSE, fig.width=8----
# Plot the densities of the distances for the anomalies
plot(distance_data, ref_cell_type = "CD8", sample_names = cd4_top6_anomalies)

## ----calculate-Sample-Distances-Similarity, warning=FALSE, message=FALSE, fig.width=8----
# Get overlap measures
overlap_measures <- calculateSampleDistancesSimilarity(query_data = query_data_subset, reference_data = ref_data_subset, 
                                                       sample_names = cd4_top6_anomalies,
                                                       query_cell_type_col = "labels", 
                                                       ref_cell_type_col = "reclustered.broad") 
overlap_measures$bhattacharyya_coef

## ----calculate-Sample-Distances-Similarity-Hellinger, warning=FALSE, message=FALSE, fig.width=8----
overlap_measures$hellinger_dist

## ----calculate-Sample-Similarity-PCA, warning=FALSE, message=FALSE, fig.width=8----
# Store PCA anomaly data and plots
anomaly_output <- detectAnomaly(reference_data = ref_data_subset, 
                                query_data = query_data_subset,
                                ref_cell_type_col = "reclustered.broad", 
                                query_cell_type_col = "labels") 
top6_anomalies <- names(sort(anomaly_output$Combined$reference_anomaly_scores, decreasing = TRUE)[1:6])

# Compute cosine similarity between anomalies and PCs
cosine_similarities <- calculateSampleSimilarityPCA(ref_data_subset, samples = top6_anomalies, 
                                                    pc_subset = c(1:50), n_top_vars = 50)
# Plot similarities
plot(cosine_similarities, pc_subset = c(15:25))

## ----project-Discriminant-Space, warning=FALSE, message=FALSE, fig.width=8----
disc_output <- calculateDiscriminantSpace(reference_data = ref_data_subset,
                                          query_data = query_data_subset, 
                                          query_cell_type_col = "labels", 
                                          ref_cell_type_col = "reclustered.broad")

# Generate scatterplot
plot(disc_output, plot_type = "scatterplot")

## ----project-Discriminant-Space-Boxplot, warning=FALSE, message=FALSE, fig.width=8----
plot(disc_output, cell_types = "CD4-CD8", plot_type = "boxplot")

## ----compare-PCA, warning=FALSE, message=FALSE, fig.width=8-------------------
# Selecting highly variable genes
ref_var <- getTopHVGs(ref_data, n = 500)
query_var <- getTopHVGs(query_data, n = 500)

# Intersect the gene symbols to obtain common genes
common_genes <- intersect(ref_var, query_var)
ref_data_subset <- ref_data[common_genes, ]
query_data_subset <- query_data[common_genes, ]

# Subset reference and query data for a specific cell type
ref_data_subset <- ref_data_subset[, which(ref_data_subset$reclustered.broad == "CD4")]
query_data_subset <- query_data_subset[, which(query_data_subset$reclustered.broad == "CD4")]

# Run PCA on the reference and query datasets separately
ref_data_subset <- runPCA(ref_data_subset, ncomponents = 50)
query_data_subset <- runPCA(query_data_subset, ncomponents = 50)

# Call the PCA comparison function
similarity_mat <- comparePCA(reference_data = ref_data_subset, query_data = query_data_subset, 
                             pc_subset = c(1:5),  
                             n_top_vars = 50,
                             metric = c("cosine", "correlation")[1], 
                             correlation_method = c("spearman", "pearson")[1])

# Create the heatmap
plot(similarity_mat)

## ----compare-PCA-Subspace, warning=FALSE, message=FALSE, fig.width=8----------
subspace_comparison <- comparePCASubspace(reference_data = ref_data_subset, query_data = query_data_subset, 
                                          pc_subset = c(1:5))

# Plot output for PCA subspace comparison
plot(subspace_comparison)

## ----compare-CCA, warning=FALSE, message=FALSE, fig.width=8-------------------
cca_comparison <- compareCCA(reference_data = ref_data_subset, query_data = query_data_subset, 
                             pc_subset = c(1:5))

# Create a data frame for plotting
plot(cca_comparison)

## ----SessionInfo, echo=FALSE, message=FALSE, warning=FALSE, comment=NA--------
options(width = 80) #reset to 'default' width

sessionInfo() #record the R and package versions used

